@@ -26,19 +26,39 @@ class PersianUtils:
             "۸": "8",
             "۹": "9",
             "۰": "0",
+            # TODO: Add arabic items too
         }
         if reverse:
             NUM_MAP = {v: k for k, v in NUM_MAP.items()}
+        else:
+            NUM_MAP.update(
+                {
+                    "١": "1",
+                    "٢": "2",
+                    "٣": "3",
+                    "٤": "4",
+                    "٥": "5",
+                    "٦": "6",
+                    "٧": "7",
+                    "٨": "8",
+                    "٩": "9",
+                    "٠": "0"
+                }
+            )
         output_string = "".join([NUM_MAP.get(c, c) for c in input_string])
         return output_string
 
     @staticmethod
-    def arabic_char2fa_char(input_string: str):
+    def arabic_char2fa_char(input_string: str, reverse: bool = False):
         arabic2persian = {
             "ك": "ک",
             "ي": "ی",
         }
-        out_string = "".join(arabic2persian.get(s, s) for s in input_string)
+        if reverse:
+            arabic2persian = {j: k for k,j in arabic2persian.items()}
+            out_string = "".join(arabic2persian.get(s, s) for s in input_string)
+        else:
+            out_string = "".join(arabic2persian.get(s, s) for s in input_string)
         return out_string
 
     @staticmethod
