@@ -2,11 +2,11 @@ import os
 import shutil
 from abc import ABC
 from os.path import join, split
-from typing import Dict, List, Union, Type
-
+from typing import Dict, List, Union, Type, Literal
+from deep_utils.utils.dir_utils.dir_utils import DirUtils
 from tqdm import tqdm
-
-from deep_utils.main_abs import MainClass
+import cv2
+from deep_utils.main_abs.main import MainClass
 from deep_utils.utils.box_utils.boxes import Box
 from deep_utils.utils.dict_named_tuple_utils import dictnamedtuple
 from deep_utils.utils.dir_utils.dir_utils import (
@@ -15,8 +15,8 @@ from deep_utils.utils.dir_utils.dir_utils import (
     transfer_directory_items,
     file_incremental
 )
-from deep_utils.utils.logging_utils import log_print
-from deep_utils.utils.opencv_utils.main import show_destroy_cv2
+from deep_utils.utils.logging_utils.logging_utils import log_print
+from deep_utils.utils.opencv_utils.opencv_utils import CVUtils
 from deep_utils.utils.os_utils.os_path import split_extension
 from deep_utils.utils.shutil_utils.shutil_utils import mv_or_copy
 from deep_utils.utils.os_utils.os_path import validate_file_extension
@@ -42,12 +42,16 @@ class YOLOObjectDetector(MainClass, ABC):
 
     @staticmethod
     def test_label_dir(dataset_dir, rename_dict: Union[Dict[int, str], None] = None, images_name="images",
-                       labels_name="labels"):
+                       labels_name="labels", show=True):
         images_path = join(dataset_dir, images_name)
         for name in sorted(os.listdir(images_path)):
             img_address = join(images_path, name)
             text_address = join(dataset_dir, labels_name, split_extension(name, '.txt'))
-            YOLOObjectDetector.test_label(img_address, text_address, rename_dict=rename_dict)
+            if show:
+                YOLOObjectDetector.test_label(img_address, text_address, rename_dict=rename_dict, show=show)
+            else:
+                img = YOLOObjectDetector.test_label(img_address, text_address, rename_dict=rename_dict, show=show)
+                yield img
 
     @staticmethod
     def test_label(img_path, label_path, rename_dict: Union[Dict[int, str], None] = None, show=True):
@@ -75,7 +79,7 @@ class YOLOObjectDetector(MainClass, ABC):
                             )
         img = Box.put_box_text(img, boxes, texts)
         if show:
-            show_destroy_cv2(img)
+            CVUtils.show_destroy_cv2(img)
         return img
 
     @staticmethod
@@ -197,6 +201,52 @@ class YOLOObjectDetector(MainClass, ABC):
             )
         return boxes, labels
 
+    @staticmethod
+    def extract_save_label(label_path, image_path, output_path: str, remove_output_path: bool = False):
+        """
+        Extract box images and save them!
+        :param label_path:
+        :param image_path:
+        :param output_path:
+        :param remove_output_path:
+        :return:
+        """
+        from deep_utils.utils.box_utils.boxes import Box
+        boxes, labels = YOLOObjectDetector.extract_label(label_path, image_path)
+        img = cv2.imread(image_path)
+        output = Box.get_box_img(img, boxes)
+        DirUtils.remove_create(output_path, remove=remove_output_path)
+        output_img = join(output_path, split(image_path)[-1])
+        for index, (box_img, lbl) in enumerate(zip(output, labels)):
+            cv2.imwrite(DirUtils.split_extension(output_img, suffix=f"_{lbl}_{index}"), box_img)
+
+    @staticmethod
+    def get_labels_images_list(label_dir: str, image_dir: str) -> List[Dict[Literal["image", "label"], str]]:
+
+        label_names = {DirUtils.split_extension(name)[0]: name for name in os.listdir(label_dir)}
+        image_names = {DirUtils.split_extension(name)[0]: name for name in os.listdir(image_dir)}
+        output = [{"label": join(label_dir, name), "image": join(image_dir, image_names[key])} for key, name in
+                  label_names.items() if key in image_names]
+        return output
+
+    @staticmethod
+    def extract_save_directory(label_dir: str, image_dir: str, output_path, remove_output_path: bool = True):
+        """
+        Extracts image boxes of a directory
+        :param label_dir:
+        :param image_dir:
+        :param output_path:
+        :param remove_output_path:
+        :return:
+        """
+        data = YOLOObjectDetector.get_labels_images_list(label_dir, image_dir)
+        DirUtils.remove_create(output_path, remove=remove_output_path)
+        for pair in data:
+            label_path = pair['label']
+            image_path = pair['image']
+            YOLOObjectDetector.extract_save_label(label_path, image_path, output_path,
+                                                  remove_output_path=False)
+
     @staticmethod
     def clean_samples(label_path, img_path, logger=None, verbose=1):
         """
@@ -343,6 +393,24 @@ class YOLOObjectDetector(MainClass, ABC):
                        verbose=1) -> Union[Type[OutputType], Dict[str, list]]:
         raise NotImplementedError("object_detects is not implemented!")
 
+    def detect_img_file(self,
+                        img_file,
+                        class_indices=None,
+                        confidence=None,
+                        iou_thresh=None,
+                        img_size=None,
+                        agnostic=None,
+                        get_time=False,
+                        logger=None,
+                        verbose=1) -> Union[Type[OutputType], Dict[str, list]]:
+        img = cv2.imread(img_file)
+        if img is None:
+            log_print(logger, f"Image {img_file} is not valid", verbose=verbose)
+        else:
+            return self.detect_objects(img, False, class_indices, confidence,
+                                       iou_thresh, img_size, agnostic, get_time,
+                                       logger, verbose)
+
     def detect_dir(self,
                    dir_,
                    confidence=None,
